/* GENERATED BY testcodegen.py; DO NOT EDIT */

#include "config.h"
#include "everything.h"

/**
 * everything_nullfunc:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_nullfunc (void)
{
  return;
}

/**
 * everything_const_return_gpointer:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
gpointer
everything_const_return_gpointer (void)
{
  return NULL;
}

/**
 * everything_const_return_gintptr:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
gintptr
everything_const_return_gintptr (void)
{
  return 0;
}

/**
 * everything_const_return_guintptr:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
guintptr
everything_const_return_guintptr (void)
{
  return 0;
}

/**
 * everything_const_return_gboolean:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gboolean
everything_const_return_gboolean (void)
{
  return 0;
}

/**
 * everything_const_return_gint8:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint8
everything_const_return_gint8 (void)
{
  return 0;
}

/**
 * everything_const_return_guint8:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint8
everything_const_return_guint8 (void)
{
  return 0;
}

/**
 * everything_const_return_gint16:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint16
everything_const_return_gint16 (void)
{
  return 0;
}

/**
 * everything_const_return_guint16:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint16
everything_const_return_guint16 (void)
{
  return 0;
}

/**
 * everything_const_return_gint32:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint32
everything_const_return_gint32 (void)
{
  return 0;
}

/**
 * everything_const_return_guint32:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint32
everything_const_return_guint32 (void)
{
  return 0;
}

/**
 * everything_const_return_gint64:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint64
everything_const_return_gint64 (void)
{
  return 0;
}

/**
 * everything_const_return_guint64:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint64
everything_const_return_guint64 (void)
{
  return 0;
}

/**
 * everything_const_return_gchar:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gchar
everything_const_return_gchar (void)
{
  return 0;
}

/**
 * everything_const_return_gshort:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gshort
everything_const_return_gshort (void)
{
  return 0;
}

/**
 * everything_const_return_gushort:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gushort
everything_const_return_gushort (void)
{
  return 0;
}

/**
 * everything_const_return_gint:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint
everything_const_return_gint (void)
{
  return 0;
}

/**
 * everything_const_return_guint:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint
everything_const_return_guint (void)
{
  return 0;
}

/**
 * everything_const_return_glong:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
glong
everything_const_return_glong (void)
{
  return 0;
}

/**
 * everything_const_return_gulong:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gulong
everything_const_return_gulong (void)
{
  return 0;
}

/**
 * everything_const_return_gsize:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gsize
everything_const_return_gsize (void)
{
  return 0;
}

/**
 * everything_const_return_gssize:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gssize
everything_const_return_gssize (void)
{
  return 0;
}

/**
 * everything_const_return_gfloat:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gfloat
everything_const_return_gfloat (void)
{
  return 0;
}

/**
 * everything_const_return_gdouble:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gdouble
everything_const_return_gdouble (void)
{
  return 0;
}

/**
 * everything_const_return_gunichar:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gunichar
everything_const_return_gunichar (void)
{
  return 0;
}

/**
 * everything_const_return_GType:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
GType
everything_const_return_GType (void)
{
  return g_object_get_type ();
}

/**
 * everything_const_return_utf8:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
const gchar*
everything_const_return_utf8 (void)
{
  return "";
}

/**
 * everything_const_return_filename:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
const gchar*
everything_const_return_filename (void)
{
  return "";
}

/**
 * everything_oneparam_gpointer:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gpointer (gpointer arg0)
{
  return;
}

/**
 * everything_oneparam_gintptr:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gintptr (gintptr arg0)
{
  return;
}

/**
 * everything_oneparam_guintptr:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_guintptr (guintptr arg0)
{
  return;
}

/**
 * everything_oneparam_gboolean:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gboolean (gboolean arg0)
{
  return;
}

/**
 * everything_oneparam_gint8:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gint8 (gint8 arg0)
{
  return;
}

/**
 * everything_oneparam_guint8:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_guint8 (guint8 arg0)
{
  return;
}

/**
 * everything_oneparam_gint16:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gint16 (gint16 arg0)
{
  return;
}

/**
 * everything_oneparam_guint16:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_guint16 (guint16 arg0)
{
  return;
}

/**
 * everything_oneparam_gint32:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gint32 (gint32 arg0)
{
  return;
}

/**
 * everything_oneparam_guint32:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_guint32 (guint32 arg0)
{
  return;
}

/**
 * everything_oneparam_gint64:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gint64 (gint64 arg0)
{
  return;
}

/**
 * everything_oneparam_guint64:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_guint64 (guint64 arg0)
{
  return;
}

/**
 * everything_oneparam_gchar:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gchar (gchar arg0)
{
  return;
}

/**
 * everything_oneparam_gshort:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gshort (gshort arg0)
{
  return;
}

/**
 * everything_oneparam_gushort:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gushort (gushort arg0)
{
  return;
}

/**
 * everything_oneparam_gint:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gint (gint arg0)
{
  return;
}

/**
 * everything_oneparam_guint:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_guint (guint arg0)
{
  return;
}

/**
 * everything_oneparam_glong:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_glong (glong arg0)
{
  return;
}

/**
 * everything_oneparam_gulong:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gulong (gulong arg0)
{
  return;
}

/**
 * everything_oneparam_gsize:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gsize (gsize arg0)
{
  return;
}

/**
 * everything_oneparam_gssize:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gssize (gssize arg0)
{
  return;
}

/**
 * everything_oneparam_gfloat:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gfloat (gfloat arg0)
{
  return;
}

/**
 * everything_oneparam_gdouble:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gdouble (gdouble arg0)
{
  return;
}

/**
 * everything_oneparam_gunichar:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_gunichar (gunichar arg0)
{
  return;
}

/**
 * everything_oneparam_GType:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_GType (GType arg0)
{
  return;
}

/**
 * everything_oneparam_utf8:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_utf8 (const gchar* arg0)
{
  return;
}

/**
 * everything_oneparam_filename:
 * @arg0:
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_oneparam_filename (const gchar* arg0)
{
  return;
}

/**
 * everything_one_outparam_gpointer:
 * @arg0: (out)  (transfer none):
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gpointer (gpointer* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gintptr:
 * @arg0: (out)  (transfer none):
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gintptr (gintptr* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_guintptr:
 * @arg0: (out)  (transfer none):
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_guintptr (guintptr* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gboolean:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gboolean (gboolean* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gint8:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gint8 (gint8* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_guint8:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_guint8 (guint8* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gint16:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gint16 (gint16* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_guint16:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_guint16 (guint16* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gint32:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gint32 (gint32* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_guint32:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_guint32 (guint32* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gint64:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gint64 (gint64* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_guint64:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_guint64 (guint64* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gchar:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gchar (gchar* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gshort:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gshort (gshort* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gushort:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gushort (gushort* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gint:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gint (gint* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_guint:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_guint (guint* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_glong:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_glong (glong* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gulong:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gulong (gulong* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gsize:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gsize (gsize* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gssize:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gssize (gssize* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gfloat:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gfloat (gfloat* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gdouble:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gdouble (gdouble* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_gunichar:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_gunichar (gunichar* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_GType:
 * @arg0: (out) :
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_GType (GType* arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_utf8:
 * @arg0: (out)  (transfer none):
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_utf8 (const gchar** arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_one_outparam_filename:
 * @arg0: (out)  (transfer none):
 *
 * Undocumented.
 */
_GI_TEST_EXTERN
void
everything_one_outparam_filename (const gchar** arg0)
{
  *arg0 = 0;
  return;
}

/**
 * everything_passthrough_one_gpointer:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
gpointer
everything_passthrough_one_gpointer (gpointer arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gintptr:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
gintptr
everything_passthrough_one_gintptr (gintptr arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_guintptr:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
guintptr
everything_passthrough_one_guintptr (guintptr arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gboolean:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gboolean
everything_passthrough_one_gboolean (gboolean arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gint8:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint8
everything_passthrough_one_gint8 (gint8 arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_guint8:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint8
everything_passthrough_one_guint8 (guint8 arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gint16:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint16
everything_passthrough_one_gint16 (gint16 arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_guint16:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint16
everything_passthrough_one_guint16 (guint16 arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gint32:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint32
everything_passthrough_one_gint32 (gint32 arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_guint32:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint32
everything_passthrough_one_guint32 (guint32 arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gint64:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint64
everything_passthrough_one_gint64 (gint64 arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_guint64:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint64
everything_passthrough_one_guint64 (guint64 arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gchar:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gchar
everything_passthrough_one_gchar (gchar arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gshort:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gshort
everything_passthrough_one_gshort (gshort arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gushort:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gushort
everything_passthrough_one_gushort (gushort arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gint:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gint
everything_passthrough_one_gint (gint arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_guint:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
guint
everything_passthrough_one_guint (guint arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_glong:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
glong
everything_passthrough_one_glong (glong arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gulong:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gulong
everything_passthrough_one_gulong (gulong arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gsize:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gsize
everything_passthrough_one_gsize (gsize arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gssize:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gssize
everything_passthrough_one_gssize (gssize arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gfloat:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gfloat
everything_passthrough_one_gfloat (gfloat arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gdouble:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gdouble
everything_passthrough_one_gdouble (gdouble arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_gunichar:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
gunichar
everything_passthrough_one_gunichar (gunichar arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_GType:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns: 
 */
_GI_TEST_EXTERN
GType
everything_passthrough_one_GType (GType arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_utf8:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
const gchar*
everything_passthrough_one_utf8 (const gchar* arg0)
{
  return arg0;
}

/**
 * everything_passthrough_one_filename:
 * @arg0:
 *
 * Undocumented.
 *
 * Returns:  (transfer none)
 */
_GI_TEST_EXTERN
const gchar*
everything_passthrough_one_filename (const gchar* arg0)
{
  return arg0;
}

